SystemOrganization addCategory: #'AL-Http'!

Object subclass: #ALConnection
	instanceVariableNames: 'in out handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALConnection class methodsFor: 'as yet unclassified' stamp: 'cwp 4/5/2011 19:15'!
in: aReadStream out: aWriteStream handler: aRequestHandler
	^ self basicNew 
		initializeWithIn: aReadStream 
		out: aWriteStream 
		handler: aRequestHandler ! !

!ALConnection class methodsFor: 'instance creation' stamp: 'cwp 4/13/2011 20:33'!
socket: aSocket handler: anObject
	^ self in: aSocket reading out: aSocket writing handler: anObject! !

!ALConnection methodsFor: 'as yet unclassified' stamp: 'cwp 4/13/2011 21:54'!
close
	in close.
	out close.
	! !

!ALConnection methodsFor: 'running' stamp: 'cwp 4/23/2011 18:56'!
handleRequest
	| parser  printer response |
	parser := ALHttpParser on: in.
	printer := ALHttpPrinter on: out.
	response := handler handleRequest: parser parseRequest.
	response header: 'Transfer-Encoding' put: 'chunked'.
	printer printResponse: response.
	response writeEntityInto: out.
	out flush.
	response isKeepAlive ifFalse: [self close]! !

!ALConnection methodsFor: 'running' stamp: 'cwp 4/13/2011 21:57'!
handleRequests
	[
		[[self handleRequest] repeat]
			on: Incomplete
			do: [:ex | self close]	
	] ifCurtailed: [self close]! !

!ALConnection methodsFor: 'initialize-release' stamp: 'cwp 4/5/2011 19:15'!
initializeWithIn: aReadStream out: aWriteStream handler: aRequestHandler
	self initialize.
	in := aReadStream.
	out := aWriteStream.
	handler := aRequestHandler ! !

Object subclass: #ALHttpParser
	instanceVariableNames: 'in ch request'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALHttpParser class methodsFor: 'as yet unclassified' stamp: 'cwp 4/5/2011 19:51'!
on: aReadStream
	^ self basicNew initializeWithStream: aReadStream! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assert: aByte
	ch = aByte ifFalse: [ALMalformedRequest signal].
	ch := nil.! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assertCrLf
	"Character cr asciiValue = 13"
	"Character lf acsiiValue = 10"
	
	self assert: 13.
	self next: 10.
! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assertSpace
	"Character space asciiValue = 32"
	self assert: 32.
! !

!ALHttpParser methodsFor: 'as yet unclassified' stamp: 'cwp 4/5/2011 19:52'!
initializeWithStream: aReadStream
	in := aReadStream
	! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 20:09'!
next: aByte
	in get = aByte ifFalse: [ALMalformedRequest signal]! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/6/2011 10:34'!
parseFieldValue
	"Character tab asciiValue = 9"
	"Character space asciiValue = 32"
	"Character cr asciivalue = 13"
	"Character lf asciiValue = 10"
	"For now, we ignore the possibility of line folding"

	| out byte |
	[#(32 9) includes: (byte := in get)] whileTrue.
	out := ByteArray new writing.
	out put: byte.
	[(byte := in get) = 13] whileFalse: [out put: byte].
	self next: 10.
	^ out conclusion asString! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 21:07'!
parseHeaders
	| name value |
	[name := self parseToken.
	name isEmpty ifTrue: [^ self assertCrLf].
	self assert: 58.
	value := self parseFieldValue.
	request header: name put: value]
		repeat
	! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 19:58'!
parseMethod
	| method |
	method := self parseToken.
	self assertSpace.
	^ method! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:00'!
parsePath
	"$/ asciiValue = 47"
	"Character space asciiValue = 32"

	| out path |
	path := ALPath root.
	ch := in get. 
	ch = 47 ifFalse: [self error].

	out := String new writing.
	[(ch := in get) = 32] whileFalse:
		[ch = 47
			ifFalse: [out put: (Character value: ch)]
			ifTrue: 
				[path := path / out conclusion.
				out := String new writing]].
	path := path / out conclusion.
	self assertSpace.
	^ path
			! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:13'!
parseRequest
	| method path version |
	method := self parseMethod.
	path := self parsePath.
	version := self parseVersion.
	request := ALRequest method: method path: path version: version.
	self parseHeaders.
	^ request! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 19:57'!
parseToken
	"These are the ascii values of characters that may be part of a token"

	|  legal out |
	legal :=  #(33 35 36 37 38 39 42 43 45 46 48 49 50 51 52 53 54 55 56 57 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 124 126).
	
	out := ByteArray new writing.
	ch ifNotNil: [out put: ch].
	[legal includes: (ch := in get)] whileTrue: [out put: ch].
	^ out conclusion asString! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:11'!
parseVersion
	| version |
	self next: $H asciiValue.
	self next: $T asciiValue.
	self next: $T asciiValue.
	self next: $P asciiValue.
	self next: $/ asciiValue.
	version := self readAll: #[48 49 50 51 52 53 54 55 56 57 46].
	self assertCrLf.
	^ version! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:12'!
readAll: aByteArray
	| out |
	out := ByteArray new writing.
	[aByteArray includes: (ch := in get)] whileTrue: [out put: ch].
	^ out conclusion asString! !

Object subclass: #ALHttpPrinter
	instanceVariableNames: 'out response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALHttpPrinter class methodsFor: 'as yet unclassified' stamp: 'cwp 4/23/2011 15:54'!
on: aWriteStream
	^ self basicNew initializeWithStream: aWriteStream ! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 4/23/2011 18:25'!
crlf
	"Character cr asciiValue -> 13"
	"Character lf asciiValue -> 10"

	out
		put: 13;
		put: 10! !

!ALHttpPrinter methodsFor: 'initialize-release' stamp: 'cwp 4/23/2011 15:55'!
initializeWithStream: aWriteStream
	out := aWriteStream  ! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 4/23/2011 18:34'!
printHeaders
	"$: asciiValue -> 58"
	"Character space asciiValue -> 32"

	response headersDo:
		[ :name :value | 
		out
			write: name;
			put: 58;
			put: 32;
			write: value.
		self crlf].
	self crlf.! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 4/23/2011 18:26'!
printResponse
	self printStatusLine.
	self printHeaders.
! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 4/23/2011 18:18'!
printResponse: aResponse
	response := aResponse.
	self printResponse.
	response := nil! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 4/23/2011 18:18'!
printStatusLine
	out
		write: 'HTTP/';
		write: (String streamContents: [ :s | response version printFractionAsDecimalOn: s ]);
		space;
		print: response status;
		space;
		write: response reason;
		cr;
		lf! !

Object subclass: #ALRequest
	instanceVariableNames: 'method path version headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 4/23/2011 16:08'!
method: aString path: aPath
	^ self method: aString path: aPath version: 'HTTP/1.1'! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 3/20/2011 21:37'!
method: aByteString path: aByteString2 version: aByteString3 
	^ self basicNew
		initializeWithMethod: aByteString
		path: aByteString2
		version: aByteString3 ! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 4/5/2011 21:39'!
at: aString
	"Answer the value of the given header."
	
	^ headers at: aString ifAbsent: [nil]! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 4/13/2011 22:42'!
header: aString put: anObject 
	headers at: aString put: anObject! !

!ALRequest methodsFor: 'initialize-release' stamp: 'cwp 4/13/2011 22:20'!
initializeWithMethod: aString path: aString2 version: aString3 
	method := aString.
	path := aString2.
	version := aString3.
	headers := Dictionary new.! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 3/21/2011 22:26'!
isConnectionKeepAlive
	^ (self at: 'Connection') = 'keep-alive'! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 3/21/2011 22:25'!
isMethodGet
	^ method = 'GET'! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 3/20/2011 22:58'!
path
	^ path! !

Object subclass: #ALRequestParser
	instanceVariableNames: 'in ch request'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALRequestParser class methodsFor: 'as yet unclassified' stamp: 'cwp 4/5/2011 19:51'!
on: aReadStream
	^ self basicNew initializeWithStream: aReadStream! !

!ALRequestParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assert: aByte
	ch = aByte ifFalse: [ALMalformedRequest signal].
	ch := nil.! !

!ALRequestParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assertCrLf
	"Character cr asciiValue = 13"
	"Character lf acsiiValue = 10"
	
	self assert: 13.
	self next: 10.
! !

!ALRequestParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assertSpace
	"Character space asciiValue = 32"
	self assert: 32.
! !

!ALRequestParser methodsFor: 'as yet unclassified' stamp: 'cwp 4/5/2011 19:52'!
initializeWithStream: aReadStream
	in := aReadStream
	! !

!ALRequestParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 20:09'!
next: aByte
	in get = aByte ifFalse: [ALMalformedRequest signal]! !

!ALRequestParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 21:38'!
parseFieldValue
	"Character tab asciiValue = 9"
	"Character space asciiValue = 32"
	"Character cr asciivalue = 13"
	"Character lf asciiValue = 10"

	| out byte |
	[#(32 9) includes: (ch := in get)] whileTrue.
	out := ByteArray new writing.
	out put: ch.
	ch := nil.
	[(byte := in get) = 13] whileFalse: [out put: byte].
	self next: 10.
	^ out conclusion asString! !

!ALRequestParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 21:07'!
parseHeaders
	| name value |
	[name := self parseToken.
	name isEmpty ifTrue: [^ self assertCrLf].
	self assert: 58.
	value := self parseFieldValue.
	request header: name put: value]
		repeat
	! !

!ALRequestParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 19:58'!
parseMethod
	| method |
	method := self parseToken.
	self assertSpace.
	^ method! !

!ALRequestParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:00'!
parsePath
	"$/ asciiValue = 47"
	"Character space asciiValue = 32"

	| out path |
	path := ALPath root.
	ch := in get. 
	ch = 47 ifFalse: [self error].

	out := String new writing.
	[(ch := in get) = 32] whileFalse:
		[ch = 47
			ifFalse: [out put: (Character value: ch)]
			ifTrue: 
				[path := path / out conclusion.
				out := String new writing]].
	path := path / out conclusion.
	self assertSpace.
	^ path
			! !

!ALRequestParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:13'!
parseRequest
	| method path version |
	method := self parseMethod.
	path := self parsePath.
	version := self parseVersion.
	request := ALRequest method: method path: path version: version.
	self parseHeaders.
	^ request! !

!ALRequestParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 19:57'!
parseToken
	"These are the ascii values of characters that may be part of a token"

	|  legal out |
	legal :=  #(33 35 36 37 38 39 42 43 45 46 48 49 50 51 52 53 54 55 56 57 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 124 126).
	
	out := ByteArray new writing.
	ch ifNotNil: [out put: ch].
	[legal includes: (ch := in get)] whileTrue: [out put: ch].
	^ out conclusion asString! !

!ALRequestParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:11'!
parseVersion
	| version |
	self next: $H asciiValue.
	self next: $T asciiValue.
	self next: $T asciiValue.
	self next: $P asciiValue.
	self next: $/ asciiValue.
	version := self readAll: #[48 49 50 51 52 53 54 55 56 57 46].
	self assertCrLf.
	^ version! !

!ALRequestParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:12'!
readAll: aByteArray
	| out |
	out := ByteArray new writing.
	[aByteArray includes: (ch := in get)] whileTrue: [out put: ch].
	^ out conclusion asString! !

Object subclass: #ALResponse
	instanceVariableNames: 'version status reason headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 4/13/2011 23:01'!
notFound: aPath
	^ self status: 404 reason: 'Not found ', aPath printString! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 4/13/2011 22:32'!
ok
	^ self status: 200 reason: 'OK'! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 4/13/2011 22:28'!
status: anInteger reason: aString
	^ self version: 1.1s1 status: anInteger reason: aString! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 4/13/2011 22:28'!
version: aScaledDecimal status: anInteger reason: aString
	^ self basicNew
		initializeWithVersion: aScaledDecimal
		status: anInteger
		reason: aString! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/13/2011 22:43'!
header: aString put: anObject
	headers at: aString put: anObject! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 18:20'!
headersDo: aBlock
	headers keysAndValuesDo: aBlock  ! !

!ALResponse methodsFor: 'initialize-release' stamp: 'cwp 4/13/2011 22:25'!
initializeWithVersion: aScaledDecimal status: anInteger reason: aString
	version := aScaledDecimal.
	status := anInteger.
	reason := aString.
	headers := Dictionary new.! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 18:44'!
isKeepAlive
	^ true! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 16:23'!
reason
	^ reason! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 16:23'!
status
	^ status! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 18:10'!
version
	^ version! !

!ALResponse methodsFor: 'as yet unclassified' stamp: 'cwp 4/23/2011 18:54'!
writeEntityInto: aWriteStream

	aWriteStream 
		write: #[53 13 10 97 97 97 97 97 13 10];
		write: #[48 13 10 13 10];
		flush! !

Object subclass: #ALServer
	instanceVariableNames: 'port process socket state application'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALServer class methodsFor: 'as yet unclassified' stamp: 'cwp 4/23/2011 17:55'!
on: anInteger application: anApplication
	^ self basicNew initializeWithPort: anInteger application: anApplication! !

!ALServer methodsFor: 'initialize-release' stamp: 'cwp 4/23/2011 17:55'!
initializeWithPort: anInteger application: anApplication
	port := anInteger.
	application := anApplication.
	! !

!ALServer methodsFor: 'testing' stamp: 'cwp 4/13/2011 17:01'!
isRunning
	^ state == #running! !

!ALServer methodsFor: 'private' stamp: 'cwp 4/13/2011 19:29'!
loop
	[state == #running] whileTrue: [self serveConnection]! !

!ALServer methodsFor: 'printing' stamp: 'cwp 4/13/2011 17:16'!
printOn: aStream
	aStream nextPutAll: (self streamingPrintOn: String new writing)! !

!ALServer methodsFor: 'private' stamp: 'cwp 4/13/2011 20:30'!
run

	socket := Socket tcpCreateIfFail: [self error: 'Unable to create a socket'].
	socket
		setOption: 'TCP_NODELAY' value: 1;
		setOption: 'SO_SNDBUF' value: 1024;
		setOption: 'SO_RCVBUF' value: 1024;
		listenOn: port backlogSize: 10.
	socket isValid ifFalse: [self error: 'Unable to listen on port ', port printString].
	
	[self loop] ensure: 
		[socket destroy.
		socket := nil.
		process := nil].

! !

!ALServer methodsFor: 'private' stamp: 'cwp 4/23/2011 17:55'!
serveConnection
	| connection |
	connection := socket waitForAcceptFor: self timeout.
	connection ifNil: [^ self].
	[
		[(ALConnection socket: connection handler: application) handleRequests] 
			forkAt: Processor highIOPriority 
			named: 'ALConnection to ', connection remoteAddress printString.
			
	] ifCurtailed: [connection destroy]! !

!ALServer methodsFor: 'running' stamp: 'cwp 4/13/2011 17:11'!
start
	self isRunning ifTrue: [self stop].	
	state := #running.
	process := [self run] forkAt: Processor lowIOPriority named: self printString.
	! !

!ALServer methodsFor: 'running' stamp: 'cwp 4/13/2011 17:05'!
stop
	state := #stopping.
	[process == nil] whileFalse: [Processor yield]! !

!ALServer methodsFor: 'printing' stamp: 'cwp 4/13/2011 17:18'!
streamingPrintOn: aWriteStream
	^ aWriteStream
		write: self class name;
		put: $[;
		print: port;
		put: $:;
		write: state;
		put: $];
		conclusion! !

!ALServer methodsFor: 'accessing' stamp: 'cwp 4/13/2011 17:07'!
timeout
	^ 5! !

Error subclass: #ALMalformedRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!
